/*
 * Created on Jun 1, 2005
 *
 * Unpublished Work, Copyright, 2005 Northrop Grumman
 */
package gov.grants.apply.util;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;

/**
 * A static utility class for common File IO operations.
 */
public final class FileUtil
{
    public static final int BUF_SIZE = 1024;
    public static final String TEMP_DIR_PREFIX = "grantsgov";

    /**
     * Deletes the directory and all files with the directory.
     * @param dir The directory to be deleted.
     * @return true if the delete was successful, false otherwise.
     */
    public static boolean deleteDir(File dir)
    {
        removeAllFiles(dir);
        return dir.delete();
    }

    /**
     * Delete given directory recursively but does not delete the directory itself.
     * @param dir The directory from which all files should be removed.
     */
    public static void removeAllFiles(File dir)
    {
        File[] files = dir.listFiles();
        for (int i = 0; files != null && i < files.length; i++)
        {
            if (files[i].isDirectory())
            {
                removeAllFiles(files[i]);
            }
            files[i].delete();
        }
    }

    /**
     * A convenience method that creates a path given a parent and child.  The
     * File.separator character is inserted if needed.
     * @param parent The parent directory.
     * @param child The child element.
     * @return The full parent/child path.
     * @see java.io.File#separator
     */
    public static String formPath(String parent, String child)
    {
        if (parent.endsWith(File.separator))
        {
            return parent + child;
        }
        else
        {
            return parent + File.separator + child;
        }
    }

    /**
     * Within the given directory, this method returns the first file it finds with
     * the given extension.  If there are multiple files with the same extension,
     * there is no gurantee which file will be returned.
     * @param searchDir The directory to search in.
     * @param ext The file extension to look for.
     * @return The file if found.
     * @exception FileNotFoundException Thrown if no files within the directory has
     *            the given extension.
     */
    public static File findFileWithExt(final File searchDir, final String ext)
        throws FileNotFoundException
    {
        return findFilesWithExt(searchDir, ext)[0];
    }

    /**
     * Within the given directory, this method returns the all files it finds with
     * the given extension.
     * @param searchDir The directory to search in.
     * @param ext The file extension to look for.
     * @return The files found.
     * @exception FileNotFoundException Thrown if no files within the directory has
     *            the given extension.
     */
    public static File[] findFilesWithExt(final File searchDir, final String ext)
        throws FileNotFoundException
    {
        File[] files = searchDir.listFiles(new FilenameFilter()
            {
                public boolean accept(File dir, String name)
                {
                    return searchDir.equals(dir) && name.endsWith(ext);
                }
            }
                                         );
        if (files != null && files.length > 0)
        {
            return files;
        }
        throw new FileNotFoundException("Could not find file ending with " + ext);
    }

    /**
     * Within the given directory, this method returns the all files it finds with
     * the given extension.
     * @param searchDir The directory to search in.
     * @param ext The file extension to look for.
     * @param exclusionExt The file extension to exclude
     * @return The files found.
     * @exception FileNotFoundException Thrown if no files within the directory has
     *            the given extension.
     */
    public static File[] findFilesWithExt(final File searchDir, final String ext, final String exclusionExt)
        throws FileNotFoundException
    {
        File[] files = searchDir.listFiles(new FilenameFilter()
            {
                public boolean accept(File dir, String name)
                {
                    return searchDir.equals(dir) && name.endsWith(ext) && !name.endsWith(exclusionExt);
                }
            }
                                         );
        if (files != null && files.length > 0)
        {
            return files;
        }
        throw new FileNotFoundException("Could not find file ending with " + ext);
    }

    /**
     * Given a file, the contents will be returned as a String.
     */
    public static String readFileIntoString(String path)
        throws FileNotFoundException, IOException
    {
        BufferedReader in = null;
        StringBuffer result = new StringBuffer();
        int charsRead = 0;
        int lastPosition = 0;
        char[] buf = new char[BUF_SIZE];
        
        try
        {
            in = new BufferedReader(new InputStreamReader
                (new FileInputStream(path)));
            while( (charsRead = in.read(buf, 0, BUF_SIZE)) > 0)
            {
                result.append(buf, 0, charsRead);
            }
            return result.toString();
        }
        finally
        {
            try 
            {
                if( in != null )
                {
                    in.close();
                }
            }
            catch(IOException e)
            {
                e.printStackTrace();
            }
        }
    }

    /**
     * Creates a directory with a random name under the given parent directory.
     * @return The path of the directory created.
     * @exception IOException Thrown if the directory could not be created.
     */
    public static File createTempDir(String parentDir)
        throws IOException
    {
        String tmpDir =
            TEMP_DIR_PREFIX + String.valueOf((int)(Math.random()*10000));

        if (parentDir != null)
        {
            tmpDir = formPath(parentDir, tmpDir);
        }

        File temp = new File(tmpDir);

        if (!temp.mkdirs() && !temp.exists() && !temp.isDirectory() && !temp.canWrite())
        {
            throw new IOException("Could not create temp directory: " + tmpDir);
        }

        temp.deleteOnExit();

        return temp;
    }

    /**
     * Pipes a given InputStream into a file created with the given file path.
     * @param outFilePath The file path to write contents to.
     * @param input The stream to read from.
     * @exception IOException Thrown with errors occur during opening of file or
     *            reading of input.
     */
    public static void readToFile(String outFilePath, InputStream input)
        throws IOException
    {
        byte[] buffer = new byte[BUF_SIZE];
        OutputStream file = null;
        int bytesRead = 0;
        try
        {
            file = new FileOutputStream(outFilePath);
            pipe(file, input);
        }
        finally
        {
            try
            {
                if (file != null) file.close();
            }
            catch (IOException e)
            {
                // ignore
            }
        }
    }

    /**
     * Pipes a given InputStream into a file created with the given file path.
     * @param outFilePath The file path to write contents to.
     * @param content The string to write into the file.
     * @exception IOException Thrown with errors occur during opening of file.
     */
    public static void writeToFile(String outFilePath, String content)
        throws IOException
    {
        ByteArrayInputStream input = new ByteArrayInputStream(content.getBytes());
        readToFile(outFilePath, input);
    }

    /**
     * Given an input and output stream, all content from the input stream is 
     * written to the output stream.
     */
    public static void pipe(OutputStream out, InputStream in)
        throws IOException
    {
        byte[] buffer = new byte[BUF_SIZE];
        int bytesRead = 0;

        try
        {
            while ((bytesRead = in.read(buffer, 0, BUF_SIZE)) > 0)
            {
                out.write(buffer, 0, bytesRead);
            }
        }
        finally
        {
            try
            {
                out.flush();
            }
            catch (IOException e)
            {
                // ignore
            }
        }
    }

    /**
     * This method will copy the file 'from' to a new file specified by 'to'
     * @param from The file to copy from.
     * @param to The file to copy to.
     */
    public static final void copy(String from, String to)
        throws IOException
    {
        copy(new File(from), new File(to));
    }

    /**
     * This method will copy the file 'from' to a new file specified by 'to'
     * @param from The file to copy from.
     * @param to The file to copy to.
     */
    public static final void copy(File from, File to)
        throws IOException
    {
        pipe(new BufferedOutputStream(new FileOutputStream(to)), new BufferedInputStream(new FileInputStream(from)));
    }

    /**
     * This method will move the file 'from' to a new file specified by 'to'
     * The 'from' file will be deleted in this move.
     * @param from The file to move from.
     * @param to The file to move to.
     */
    public static final void move(String from, String to)
        throws IOException
    {
        copy(from, to);
        new File(from).delete();
    }

    /**
     * This method will move the file 'from' to a new file specified by 'to'
     * The 'from' file will be deleted in this move.
     * @param from The file to move from.
     * @param to The file to move to.
     */
    public static final void move(File from, File to)
        throws IOException
    {
        copy(from, to);
        from.delete();
    }
/*
    public static final void replaceString(String filePath, String oldString, String newString)
        throws FileNotFoundException, IOException
    {
        String file = readFileIntoString(filePath);
        String newFile = StringUtils.strReplace(file, oldString, newString);
        writeToFile(filePath, newFile);
    }
    */
}

