/*
 * Created on Jun 3, 2005
 *
 * Unpublished Work, Copyright, 2005 Northrop Grumman
 */
package gov.grants.apply.util;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.HashMap;

import org.apache.log4j.Logger;


/**
 * @author Grants.gov WebServices Team
 *
 * Utility class specifically for timing operations and logging them at debug level.
 */
public final class PerformanceTimer
{
   private static PerformanceTimer _instance = null;
   private static final Logger LOG = Logger.getLogger(PerformanceTimer.class);
   
   private HashMap _startTimes = null;
   
   private PerformanceTimer()
   {
      _startTimes = new HashMap();
   }
   
   /**
    * @return The singleton instance.
    */
   public static PerformanceTimer getInstance()
   {
      if (_instance == null)
      {
         _instance = new PerformanceTimer();
      }
      return _instance;
   }

   /**
    * Starts the timer for the current thread.
    */
   public static void start()
   {
      getInstance().start(Thread.currentThread());
   }
   
   /**
    * Prints the duration of time between the start of the timer and the current time.
    * This does not reset the start time.
    */
   public static void stop()
   {
      getInstance().stop(Thread.currentThread());
   }
   
   /**
    * Setting the start time identified by the thread who is setting it.
    * We set the time at the last possible millisecond since looking up the map, casting,
    * and creating the StartTime object (only if needed) will take up precious time.
    * Note that this method only runs if the Logger is set to debug.
    * @param owner The thread performing the operation that you want to time.
    */
   public void start(Thread owner)
   {
      if (LOG.isDebugEnabled())
      {
         StartTime time = (StartTime) _startTimes.get(owner);
         if (time == null)
         {
            time = new StartTime();
            _startTimes.put(owner, time);
         }
         time.setTime(System.currentTimeMillis());
      }
   }

   /**
    * Logs the time it took the last operation to complete.  This method only runs when
    * the Logger is set to the debug level.  This does not reset the start time.
    * @param owner The thread performing the operation that you want to time.
    */
   public void stop(Thread owner)
   {
      if (LOG.isDebugEnabled())
      {
         long stop = System.currentTimeMillis();
         StartTime start = (StartTime) _startTimes.get(owner);
         if (start == null)
         {
            LOG.debug("The '" + owner.getName() + " thread never started a time.");
         }
         else
         {
            long time = stop - start.getTime();
            /* Can only be used by jdk 1.4 code.
            StackTraceElement[] stack = new Exception().getStackTrace();
            StackTraceElement caller = null;
            String thisClassName = getClass().getName();
            for (int i = 0; i < stack.length; i++)
            {
               if (!thisClassName.equals(stack[i].getClassName()))
               {
                  caller = stack[i];
                  break;
               }
            }
            LOG.debug(caller.getClassName() + "." + caller.getMethodName() + "() on thread '" + owner.getName() + "' took " + time + " milliseconds.");
            */
            ThrowableTraceTextParser t = new ThrowableTraceTextParser(new Exception());
            LOG.debug(t.getLastCall() + " on thread '" + owner.getName() + "' took " + time + " milliseconds.");
         }
      }
   }

   private static final class ThrowableTraceTextParser
   {
      private String _lastCall = null;
      
      private ThrowableTraceTextParser(Throwable t)
      {
         StringWriter writer = new StringWriter();
         t.printStackTrace(new PrintWriter(writer));
         String trace = writer.toString();
         int start = trace.lastIndexOf("PerformanceTimer.java");
         start = trace.indexOf("at ", start) + "at ".length();
         int end = trace.indexOf(')', start) + 1;
         _lastCall = trace.substring(start, end);
      }
      
      private String getLastCall()
      {
         return _lastCall;
      }
   }
   
   private static final class StartTime
   {
      private long _time = 0;

      public void setTime(long time)
      {
         _time = time;
      }
      
      public long getTime()
      {
         return _time;
      }
   }
}
